@@ -1,9 +1,5 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Virgile Andreani <virgile.andreani@anbuco.fr>
-// (c) kwantam <kwantam@gmail.com>
-//     * 2015-04-28 ~ updated to work with both UTF-8 and non-UTF-8 encodings
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
@@ -373,6 +369,7 @@ enum CharType {
     Other,
 }
 
+#[allow(clippy::cognitive_complexity)]
 fn expand(options: &Options) -> std::io::Result<()> {
     use self::CharType::*;
 
