@@ -1,19 +1,16 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 #![allow(unused_must_use)] // because we of writeln!
 
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Inokentiy Babushkin <inokentiy.babushkin@googlemail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE file
-//  * that was distributed with this source code.
-
 // spell-checker:ignore (ToDO) lstat
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
 use std::fs;
 use std::io::{ErrorKind, Write};
 use uucore::display::Quotable;
 use uucore::error::{set_exit_code, UResult, UUsageError};
-use uucore::format_usage;
+use uucore::{format_usage, help_about, help_usage};
 
 // operating mode
 enum Mode {
@@ -23,8 +20,8 @@ enum Mode {
     Both,    // a combination of `Basic` and `Extra`
 }
 
-static ABOUT: &str = "Check whether file names are valid or portable";
-const USAGE: &str = "{} [OPTION]... NAME...";
+const ABOUT: &str = help_about!("pathchk.md");
+const USAGE: &str = help_usage!("pathchk.md");
 
 mod options {
     pub const POSIX: &str = "posix";
@@ -44,9 +41,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let matches = uu_app().try_get_matches_from(args)?;
 
     // set working mode
-    let is_posix = matches.get_many::<String>(options::POSIX).is_some();
-    let is_posix_special = matches.get_many::<String>(options::POSIX_SPECIAL).is_some();
-    let is_portability = matches.get_many::<String>(options::PORTABILITY).is_some();
+    let is_posix = matches.get_flag(options::POSIX);
+    let is_posix_special = matches.get_flag(options::POSIX_SPECIAL);
+    let is_portability = matches.get_flag(options::PORTABILITY);
 
     let mode = if (is_posix && is_posix_special) || is_portability {
         Mode::Both
@@ -82,7 +79,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     Ok(())
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -91,22 +88,25 @@ pub fn uu_app<'a>() -> Command<'a> {
         .arg(
             Arg::new(options::POSIX)
                 .short('p')
-                .help("check for most POSIX systems"),
+                .help("check for most POSIX systems")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::POSIX_SPECIAL)
                 .short('P')
-                .help(r#"check for empty names and leading "-""#),
+                .help(r#"check for empty names and leading "-""#)
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::PORTABILITY)
                 .long(options::PORTABILITY)
-                .help("check for all POSIX systems (equivalent to -p -P)"),
+                .help("check for all POSIX systems (equivalent to -p -P)")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::PATH)
                 .hide(true)
-                .multiple_occurrences(true)
+                .action(ArgAction::Append)
                 .value_hint(clap::ValueHint::AnyPath),
         )
 }
@@ -129,10 +129,7 @@ fn check_basic(path: &[String]) -> bool {
     if total_len > POSIX_PATH_MAX {
         writeln!(
             std::io::stderr(),
-            "limit {} exceeded by length {} of file name {}",
-            POSIX_PATH_MAX,
-            total_len,
-            joined_path
+            "limit {POSIX_PATH_MAX} exceeded by length {total_len} of file name {joined_path}"
         );
         return false;
     } else if total_len == 0 {
@@ -223,7 +220,7 @@ fn check_searchable(path: &str) -> bool {
             if e.kind() == ErrorKind::NotFound {
                 true
             } else {
-                writeln!(std::io::stderr(), "{}", e);
+                writeln!(std::io::stderr(), "{e}");
                 false
             }
         }
