@@ -1,7 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore (ToDO) istr chiter argptr ilen extendedbigdecimal extendedbigint numberparse
 use std::io::{stdout, ErrorKind, Write};
 use std::process::exit;
@@ -31,7 +31,7 @@ const USAGE: &str = help_usage!("seq.md");
 
 const OPT_SEPARATOR: &str = "separator";
 const OPT_TERMINATOR: &str = "terminator";
-const OPT_WIDTHS: &str = "widths";
+const OPT_EQUAL_WIDTH: &str = "equal-width";
 const OPT_FORMAT: &str = "format";
 
 const ARG_NUMBERS: &str = "numbers";
@@ -40,7 +40,7 @@ const ARG_NUMBERS: &str = "numbers";
 struct SeqOptions<'a> {
     separator: String,
     terminator: String,
-    widths: bool,
+    equal_width: bool,
     format: Option<&'a str>,
 }
 
@@ -58,10 +58,13 @@ type RangeFloat = (ExtendedBigDecimal, ExtendedBigDecimal, ExtendedBigDecimal);
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let matches = uu_app().try_get_matches_from(args)?;
 
-    let numbers = matches
-        .get_many::<String>(ARG_NUMBERS)
-        .unwrap()
-        .collect::<Vec<_>>();
+    let numbers_option = matches.get_many::<String>(ARG_NUMBERS);
+
+    if numbers_option.is_none() {
+        return Err(SeqError::NoArguments.into());
+    }
+
+    let numbers = numbers_option.unwrap().collect::<Vec<_>>();
 
     let options = SeqOptions {
         separator: matches
@@ -74,7 +77,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             .map(|s| s.as_str())
             .unwrap_or("\n")
             .to_string(),
-        widths: matches.get_flag(OPT_WIDTHS),
+        equal_width: matches.get_flag(OPT_EQUAL_WIDTH),
         format: matches.get_one::<String>(OPT_FORMAT).map(|s| s.as_str()),
     };
 
@@ -123,7 +126,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 (first, increment, last),
                 &options.separator,
                 &options.terminator,
-                options.widths,
+                options.equal_width,
                 padding,
                 options.format,
             )
@@ -137,7 +140,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             largest_dec,
             &options.separator,
             &options.terminator,
-            options.widths,
+            options.equal_width,
             padding,
             options.format,
         ),
@@ -170,9 +173,9 @@ pub fn uu_app() -> Command {
                 .help("Terminator character (defaults to \\n)"),
         )
         .arg(
-            Arg::new(OPT_WIDTHS)
+            Arg::new(OPT_EQUAL_WIDTH)
                 .short('w')
-                .long("widths")
+                .long("equal-width")
                 .help("Equalize widths of all numbers by padding with zeros")
                 .action(ArgAction::SetTrue),
         )
@@ -203,7 +206,6 @@ fn write_value_float(
     value: &ExtendedBigDecimal,
     width: usize,
     precision: usize,
-    _is_first_iteration: bool,
 ) -> std::io::Result<()> {
     let value_as_str =
         if *value == ExtendedBigDecimal::Infinity || *value == ExtendedBigDecimal::MinusInfinity {
@@ -220,16 +222,13 @@ fn write_value_int(
     value: &ExtendedBigInt,
     width: usize,
     pad: bool,
-    is_first_iteration: bool,
 ) -> std::io::Result<()> {
     let value_as_str = if pad {
-        if *value == ExtendedBigInt::MinusZero && is_first_iteration {
-            format!("-{value:>0width$}", width = width - 1)
+        if *value == ExtendedBigInt::MinusZero {
+            format!("{value:0<width$}")
         } else {
             format!("{value:>0width$}")
         }
-    } else if *value == ExtendedBigInt::MinusZero && is_first_iteration {
-        format!("-{value}")
     } else {
         format!("{value}")
     };
@@ -279,13 +278,7 @@ fn print_seq(
                     exit(1);
                 }
             }
-            None => write_value_float(
-                &mut stdout,
-                &value,
-                padding,
-                largest_dec,
-                is_first_iteration,
-            )?,
+            None => write_value_float(&mut stdout, &value, padding, largest_dec)?,
         }
         // TODO Implement augmenting addition.
         value = value + increment.clone();
@@ -344,7 +337,7 @@ fn print_seq_integers(
                     exit(1);
                 }
             }
-            None => write_value_int(&mut stdout, &value, padding, pad, is_first_iteration)?,
+            None => write_value_int(&mut stdout, &value, padding, pad)?,
         }
         // TODO Implement augmenting addition.
         value = value + increment.clone();
