@@ -1,9 +1,7 @@
-// * This file is part of the uutils coreutils package.
-// *
-// * (c) T. Jameson Little <t.jameson.little@gmail.com>
-// *
-// * For the full copyright and license information, please view the LICENSE file
-// * that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) sysv
 
@@ -13,11 +11,10 @@ use std::io::{stdin, Read};
 use std::path::Path;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError};
-use uucore::{format_usage, show};
+use uucore::{format_usage, help_about, help_usage, show};
 
-static USAGE: &str = "{} [OPTION]... [FILE]...";
-static ABOUT: &str = "Checksum and count the blocks in a file.\n\n\
-                      With no FILE, or when FILE is -, read standard input.";
+const USAGE: &str = help_usage!("sum.md");
+const ABOUT: &str = help_about!("sum.md");
 
 // This can be replaced with usize::div_ceil once it is stabilized.
 // This implementation approach is optimized for when `b` is a constant,
@@ -34,8 +31,8 @@ fn bsd_sum(mut reader: Box<dyn Read>) -> (usize, u16) {
         match reader.read(&mut buf) {
             Ok(n) if n != 0 => {
                 bytes_read += n;
-                for &byte in buf[..n].iter() {
-                    checksum = (checksum >> 1) + ((checksum & 1) << 15);
+                for &byte in &buf[..n] {
+                    checksum = checksum.rotate_right(1);
                     checksum = checksum.wrapping_add(u16::from(byte));
                 }
             }
@@ -57,7 +54,7 @@ fn sysv_sum(mut reader: Box<dyn Read>) -> (usize, u16) {
         match reader.read(&mut buf) {
             Ok(n) if n != 0 => {
                 bytes_read += n;
-                for &byte in buf[..n].iter() {
+                for &byte in &buf[..n] {
                     ret = ret.wrapping_add(u32::from(byte));
                 }
             }
