@@ -10,9 +10,9 @@ use crate::{parse, platform, Quotable};
 use clap::{crate_version, value_parser};
 use clap::{Arg, ArgAction, ArgMatches, Command};
 use fundu::{DurationParser, SaturatingInto};
-use is_terminal::IsTerminal;
 use same_file::Handle;
 use std::ffi::OsString;
+use std::io::IsTerminal;
 use std::time::Duration;
 use uucore::error::{UResult, USimpleError, UUsageError};
 use uucore::parse_size::{parse_size, ParseSizeError};
