@@ -1,11 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Morten Olsen Lysgaard <morten@lysgaard.no>
-//  * (c) Alexander Batischev <eual.jp@gmail.com>
-//  * (c) Thomas Queiroz <thomasqueirozb@gmail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) seekable seek'd tail'ing ringbuffer ringbuf unwatch Uncategorized filehandle Signum
 // spell-checker:ignore (libs) kqueue
@@ -16,13 +12,6 @@
 // spell-checker:ignore (shell/tools)
 // spell-checker:ignore (misc)
 
-#[macro_use]
-extern crate clap;
-
-#[macro_use]
-extern crate uucore;
-extern crate core;
-
 pub mod args;
 pub mod chunks;
 mod follow;
@@ -31,57 +20,57 @@ mod paths;
 mod platform;
 pub mod text;
 
+pub use args::uu_app;
+use args::{parse_args, FilterMode, Settings, Signum};
+use chunks::ReverseChunks;
+use follow::Observer;
+use paths::{FileExtTail, HeaderPrinter, Input, InputKind, MetadataExtTail};
 use same_file::Handle;
 use std::cmp::Ordering;
 use std::fs::File;
 use std::io::{self, stdin, stdout, BufRead, BufReader, BufWriter, Read, Seek, SeekFrom, Write};
 use std::path::{Path, PathBuf};
-
 use uucore::display::Quotable;
 use uucore::error::{get_exit_code, set_exit_code, FromIo, UError, UResult, USimpleError};
-
-pub use args::uu_app;
-use args::{parse_args, FilterMode, Settings, Signum};
-use chunks::ReverseChunks;
-use follow::WatcherService;
-use paths::{FileExtTail, Input, InputKind, InputService, MetadataExtTail};
+use uucore::{show, show_error};
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let settings = parse_args(args)?;
+
+    settings.check_warnings();
+
+    match settings.verify() {
+        args::VerificationResult::CannotFollowStdinByName => {
+            return Err(USimpleError::new(
+                1,
+                format!("cannot follow {} by name", text::DASH.quote()),
+            ))
+        }
+        // Exit early if we do not output anything. Note, that this may break a pipe
+        // when tail is on the receiving side.
+        args::VerificationResult::NoOutput => return Ok(()),
+        args::VerificationResult::Ok => {}
+    }
+
     uu_tail(&settings)
 }
 
 fn uu_tail(settings: &Settings) -> UResult<()> {
-    // Mimic GNU's tail for `tail -F` and exit immediately
-    let mut input_service = InputService::from(settings);
-    let mut watcher_service = WatcherService::from(settings);
+    let mut printer = HeaderPrinter::new(settings.verbose, true);
+    let mut observer = Observer::from(settings);
 
-    if input_service.has_stdin() && watcher_service.follow_name() {
-        return Err(USimpleError::new(
-            1,
-            format!("cannot follow {} by name", text::DASH.quote()),
-        ));
-    }
-
-    watcher_service.start(settings)?;
+    observer.start(settings)?;
     // Do an initial tail print of each path's content.
     // Add `path` and `reader` to `files` map if `--follow` is selected.
-    for input in &input_service.inputs.clone() {
+    for input in &settings.inputs.clone() {
         match input.kind() {
             InputKind::File(path) if cfg!(not(unix)) || path != &PathBuf::from(text::DEV_STDIN) => {
-                tail_file(
-                    settings,
-                    &mut input_service,
-                    input,
-                    path,
-                    &mut watcher_service,
-                    0,
-                )?;
+                tail_file(settings, &mut printer, input, path, &mut observer, 0)?;
             }
             // File points to /dev/stdin here
             InputKind::File(_) | InputKind::Stdin => {
-                tail_stdin(settings, &mut input_service, input, &mut watcher_service)?;
+                tail_stdin(settings, &mut printer, input, &mut observer)?;
             }
         }
     }
@@ -96,9 +85,8 @@ fn uu_tail(settings: &Settings) -> UResult<()> {
         the input file is not a FIFO, pipe, or regular file, it is unspecified whether or
         not the -f option shall be ignored.
         */
-
-        if !input_service.has_only_stdin() {
-            follow::follow(watcher_service, settings)?;
+        if !settings.has_only_stdin() {
+            follow::follow(observer, settings)?;
         }
     }
 
@@ -111,16 +99,12 @@ fn uu_tail(settings: &Settings) -> UResult<()> {
 
 fn tail_file(
     settings: &Settings,
-    input_service: &mut InputService,
+    header_printer: &mut HeaderPrinter,
     input: &Input,
     path: &Path,
-    watcher_service: &mut WatcherService,
+    observer: &mut Observer,
     offset: u64,
 ) -> UResult<()> {
-    if watcher_service.follow_descriptor_retry() {
-        show_warning!("--retry only effective for the initial open");
-    }
-
     if !path.exists() {
         set_exit_code(1);
         show_error!(
@@ -128,19 +112,19 @@ fn tail_file(
             input.display_name,
             text::NO_SUCH_FILE
         );
-        watcher_service.add_bad_path(path, input.display_name.as_str(), false)?;
+        observer.add_bad_path(path, input.display_name.as_str(), false)?;
     } else if path.is_dir() {
         set_exit_code(1);
 
-        input_service.print_header(input);
+        header_printer.print_input(input);
         let err_msg = "Is a directory".to_string();
 
         show_error!("error reading '{}': {}", input.display_name, err_msg);
         if settings.follow.is_some() {
-            let msg = if !settings.retry {
-                "; giving up on this name"
-            } else {
+            let msg = if settings.retry {
                 ""
+            } else {
+                "; giving up on this name"
             };
             show_error!(
                 "{}: cannot follow end of this type of file{}",
@@ -148,16 +132,16 @@ fn tail_file(
                 msg
             );
         }
-        if !(watcher_service.follow_name_retry()) {
+        if !(observer.follow_name_retry()) {
             // skip directory if not retry
             return Ok(());
         }
-        watcher_service.add_bad_path(path, input.display_name.as_str(), false)?;
+        observer.add_bad_path(path, input.display_name.as_str(), false)?;
     } else if input.is_tailable() {
         let metadata = path.metadata().ok();
-        match File::open(&path) {
+        match File::open(path) {
             Ok(mut file) => {
-                input_service.print_header(input);
+                header_printer.print_input(input);
                 let mut reader;
                 if !settings.presume_input_pipe
                     && file.is_seekable(if input.is_stdin() { offset } else { 0 })
@@ -169,7 +153,7 @@ fn tail_file(
                     reader = BufReader::new(file);
                     unbounded_tail(&mut reader, settings)?;
                 }
-                watcher_service.add_path(
+                observer.add_path(
                     path,
                     input.display_name.as_str(),
                     Some(Box::new(reader)),
@@ -177,20 +161,20 @@ fn tail_file(
                 )?;
             }
             Err(e) if e.kind() == std::io::ErrorKind::PermissionDenied => {
-                watcher_service.add_bad_path(path, input.display_name.as_str(), false)?;
+                observer.add_bad_path(path, input.display_name.as_str(), false)?;
                 show!(e.map_err_context(|| {
                     format!("cannot open '{}' for reading", input.display_name)
                 }));
             }
             Err(e) => {
-                watcher_service.add_bad_path(path, input.display_name.as_str(), false)?;
+                observer.add_bad_path(path, input.display_name.as_str(), false)?;
                 return Err(e.map_err_context(|| {
                     format!("cannot open '{}' for reading", input.display_name)
                 }));
             }
         }
     } else {
-        watcher_service.add_bad_path(path, input.display_name.as_str(), false)?;
+        observer.add_bad_path(path, input.display_name.as_str(), false)?;
     }
 
     Ok(())
@@ -198,9 +182,9 @@ fn tail_file(
 
 fn tail_stdin(
     settings: &Settings,
-    input_service: &mut InputService,
+    header_printer: &mut HeaderPrinter,
     input: &Input,
-    watcher_service: &mut WatcherService,
+    observer: &mut Observer,
 ) -> UResult<()> {
     match input.resolve() {
         // fifo
@@ -210,32 +194,24 @@ fn tail_stdin(
                 // Save the current seek position/offset of a stdin redirected file.
                 // This is needed to pass "gnu/tests/tail-2/start-middle.sh"
                 if let Ok(mut stdin_handle) = Handle::stdin() {
-                    if let Ok(offset) = stdin_handle.as_file_mut().seek(SeekFrom::Current(0)) {
+                    if let Ok(offset) = stdin_handle.as_file_mut().stream_position() {
                         stdin_offset = offset;
                     }
                 }
             }
             tail_file(
                 settings,
-                input_service,
+                header_printer,
                 input,
                 &path,
-                watcher_service,
+                observer,
                 stdin_offset,
             )?;
         }
         // pipe
         None => {
-            input_service.print_header(input);
-            if !paths::stdin_is_bad_fd() {
-                let mut reader = BufReader::new(stdin());
-                unbounded_tail(&mut reader, settings)?;
-                watcher_service.add_stdin(
-                    input.display_name.as_str(),
-                    Some(Box::new(reader)),
-                    true,
-                )?;
-            } else {
+            header_printer.print_input(input);
+            if paths::stdin_is_bad_fd() {
                 set_exit_code(1);
                 show_error!(
                     "cannot fstat {}: {}",
@@ -249,6 +225,10 @@ fn tail_stdin(
                         text::BAD_FD
                     );
                 }
+            } else {
+                let mut reader = BufReader::new(stdin());
+                unbounded_tail(&mut reader, settings)?;
+                observer.add_stdin(input.display_name.as_str(), Some(Box::new(reader)), true)?;
             }
         }
     };
@@ -423,7 +403,7 @@ fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UR
         FilterMode::Lines(Signum::Negative(count), sep) => {
             let mut chunks = chunks::LinesChunkBuffer::new(*sep, *count);
             chunks.fill(reader)?;
-            chunks.print(writer)?;
+            chunks.print(&mut writer)?;
         }
         FilterMode::Lines(Signum::PlusZero | Signum::Positive(1), _) => {
             io::copy(reader, &mut writer)?;
@@ -447,7 +427,7 @@ fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UR
         FilterMode::Bytes(Signum::Negative(count)) => {
             let mut chunks = chunks::BytesChunkBuffer::new(*count);
             chunks.fill(reader)?;
-            chunks.print(writer)?;
+            chunks.print(&mut writer)?;
         }
         FilterMode::Bytes(Signum::PlusZero | Signum::Positive(1)) => {
             io::copy(reader, &mut writer)?;
